// Copyright (c) 2016-2025 Tobias Junghans <tobydox@veyon.io>
// This file is part of Veyon - https://veyon.io
// SPDX-License-Identifier: LGPL-2.0-or-later

#include "ldap_core_debug.h"

Q_LOGGING_CATEGORY(LDAP_LOG, "KLDAP");
