// Copyright (c) 2019-2025 Tobias Junghans <tobydox@veyon.io>
// This file is part of Veyon - https://veyon.io
// SPDX-License-Identifier: LGPL-2.0-or-later

#include "VeyonConfiguration.h"
#include "LdapConfiguration.h"

IMPLEMENT_CONFIG_PROXY(LdapConfiguration)
