// Copyright (c) 2019-2025 Tobias Junghans <tobydox@veyon.io>
// This file is part of Veyon - https://veyon.io
// SPDX-License-Identifier: LGPL-2.0-or-later

#pragma once

#if defined(ldap_common_EXPORTS)
#define LDAP_COMMON_EXPORT Q_DECL_EXPORT
#else
#define LDAP_COMMON_EXPORT Q_DECL_IMPORT
#endif

